@@ -15,7 +15,9 @@
 # specific language governing permissions and limitations
 # under the License.
 
-from typing import Dict, List, Optional, Union
+import threading
+from dataclasses import dataclass
+from typing import Any, Callable, Optional, Union
 
 from selenium.webdriver.common.bidi.common import command_builder
 
@@ -55,7 +57,7 @@ class NavigationInfo:
         self.url = url
 
     @classmethod
-    def from_json(cls, json: Dict) -> "NavigationInfo":
+    def from_json(cls, json: dict) -> "NavigationInfo":
         """Creates a NavigationInfo instance from a dictionary.
 
         Parameters:
@@ -66,12 +68,23 @@ class NavigationInfo:
         -------
             NavigationInfo: A new instance of NavigationInfo.
         """
-        return cls(
-            context=json.get("context"),
-            navigation=json.get("navigation"),
-            timestamp=json.get("timestamp"),
-            url=json.get("url"),
-        )
+        context = json.get("context")
+        if context is None or not isinstance(context, str):
+            raise ValueError("context is required and must be a string")
+
+        navigation = json.get("navigation")
+        if navigation is not None and not isinstance(navigation, str):
+            raise ValueError("navigation must be a string")
+
+        timestamp = json.get("timestamp")
+        if timestamp is None or not isinstance(timestamp, int) or timestamp < 0:
+            raise ValueError("timestamp is required and must be a non-negative integer")
+
+        url = json.get("url")
+        if url is None or not isinstance(url, str):
+            raise ValueError("url is required and must be a string")
+
+        return cls(context, navigation, timestamp, url)
 
 
 class BrowsingContextInfo:
@@ -81,11 +94,11 @@ class BrowsingContextInfo:
         self,
         context: str,
         url: str,
-        children: Optional[List["BrowsingContextInfo"]],
+        children: Optional[list["BrowsingContextInfo"]],
+        client_window: str,
+        user_context: str,
         parent: Optional[str] = None,
-        user_context: Optional[str] = None,
         original_opener: Optional[str] = None,
-        client_window: Optional[str] = None,
     ):
         self.context = context
         self.url = url
@@ -96,7 +109,7 @@ class BrowsingContextInfo:
         self.client_window = client_window
 
     @classmethod
-    def from_json(cls, json: Dict) -> "BrowsingContextInfo":
+    def from_json(cls, json: dict) -> "BrowsingContextInfo":
         """Creates a BrowsingContextInfo instance from a dictionary.
 
         Parameters:
@@ -108,17 +121,49 @@ class BrowsingContextInfo:
             BrowsingContextInfo: A new instance of BrowsingContextInfo.
         """
         children = None
-        if json.get("children") is not None:
-            children = [BrowsingContextInfo.from_json(child) for child in json.get("children")]
+        raw_children = json.get("children")
+        if raw_children is not None:
+            if not isinstance(raw_children, list):
+                raise ValueError("children must be a list if provided")
+
+            children = []
+            for child in raw_children:
+                if not isinstance(child, dict):
+                    raise ValueError(f"Each child must be a dictionary, got {type(child)}")
+                children.append(BrowsingContextInfo.from_json(child))
+
+        context = json.get("context")
+        if context is None or not isinstance(context, str):
+            raise ValueError("context is required and must be a string")
+
+        url = json.get("url")
+        if url is None or not isinstance(url, str):
+            raise ValueError("url is required and must be a string")
+
+        parent = json.get("parent")
+        if parent is not None and not isinstance(parent, str):
+            raise ValueError("parent must be a string if provided")
+
+        user_context = json.get("userContext")
+        if user_context is None or not isinstance(user_context, str):
+            raise ValueError("userContext is required and must be a string")
+
+        original_opener = json.get("originalOpener")
+        if original_opener is not None and not isinstance(original_opener, str):
+            raise ValueError("originalOpener must be a string if provided")
+
+        client_window = json.get("clientWindow")
+        if client_window is None or not isinstance(client_window, str):
+            raise ValueError("clientWindow is required and must be a string")
 
         return cls(
-            context=json.get("context"),
-            url=json.get("url"),
+            context=context,
+            url=url,
             children=children,
-            parent=json.get("parent"),
-            user_context=json.get("userContext"),
-            original_opener=json.get("originalOpener"),
-            client_window=json.get("clientWindow"),
+            client_window=client_window,
+            user_context=user_context,
+            parent=parent,
+            original_opener=original_opener,
         )
 
 
@@ -137,7 +182,7 @@ class DownloadWillBeginParams(NavigationInfo):
         self.suggested_filename = suggested_filename
 
     @classmethod
-    def from_json(cls, json: Dict) -> "DownloadWillBeginParams":
+    def from_json(cls, json: dict) -> "DownloadWillBeginParams":
         """Creates a DownloadWillBeginParams instance from a dictionary.
 
         Parameters:
@@ -148,12 +193,32 @@ class DownloadWillBeginParams(NavigationInfo):
         -------
             DownloadWillBeginParams: A new instance of DownloadWillBeginParams.
         """
+        context = json.get("context")
+        if context is None or not isinstance(context, str):
+            raise ValueError("context is required and must be a string")
+
+        navigation = json.get("navigation")
+        if navigation is not None and not isinstance(navigation, str):
+            raise ValueError("navigation must be a string")
+
+        timestamp = json.get("timestamp")
+        if timestamp is None or not isinstance(timestamp, int) or timestamp < 0:
+            raise ValueError("timestamp is required and must be a non-negative integer")
+
+        url = json.get("url")
+        if url is None or not isinstance(url, str):
+            raise ValueError("url is required and must be a string")
+
+        suggested_filename = json.get("suggestedFilename")
+        if suggested_filename is None or not isinstance(suggested_filename, str):
+            raise ValueError("suggestedFilename is required and must be a string")
+
         return cls(
-            context=json.get("context"),
-            navigation=json.get("navigation"),
-            timestamp=json.get("timestamp"),
-            url=json.get("url"),
-            suggested_filename=json.get("suggestedFilename"),
+            context=context,
+            navigation=navigation,
+            timestamp=timestamp,
+            url=url,
+            suggested_filename=suggested_filename,
         )
 
 
@@ -175,7 +240,7 @@ class UserPromptOpenedParams:
         self.default_value = default_value
 
     @classmethod
-    def from_json(cls, json: Dict) -> "UserPromptOpenedParams":
+    def from_json(cls, json: dict) -> "UserPromptOpenedParams":
         """Creates a UserPromptOpenedParams instance from a dictionary.
 
         Parameters:
@@ -186,12 +251,32 @@ class UserPromptOpenedParams:
         -------
             UserPromptOpenedParams: A new instance of UserPromptOpenedParams.
         """
+        context = json.get("context")
+        if context is None or not isinstance(context, str):
+            raise ValueError("context is required and must be a string")
+
+        handler = json.get("handler")
+        if handler is None or not isinstance(handler, str):
+            raise ValueError("handler is required and must be a string")
+
+        message = json.get("message")
+        if message is None or not isinstance(message, str):
+            raise ValueError("message is required and must be a string")
+
+        type_value = json.get("type")
+        if type_value is None or not isinstance(type_value, str):
+            raise ValueError("type is required and must be a string")
+
+        default_value = json.get("defaultValue")
+        if default_value is not None and not isinstance(default_value, str):
+            raise ValueError("defaultValue must be a string if provided")
+
         return cls(
-            context=json.get("context"),
-            handler=json.get("handler"),
-            message=json.get("message"),
-            type=json.get("type"),
-            default_value=json.get("defaultValue"),
+            context=context,
+            handler=handler,
+            message=message,
+            type=type_value,
+            default_value=default_value,
         )
 
 
@@ -211,7 +296,7 @@ class UserPromptClosedParams:
         self.user_text = user_text
 
     @classmethod
-    def from_json(cls, json: Dict) -> "UserPromptClosedParams":
+    def from_json(cls, json: dict) -> "UserPromptClosedParams":
         """Creates a UserPromptClosedParams instance from a dictionary.
 
         Parameters:
@@ -222,11 +307,27 @@ class UserPromptClosedParams:
         -------
             UserPromptClosedParams: A new instance of UserPromptClosedParams.
         """
+        context = json.get("context")
+        if context is None or not isinstance(context, str):
+            raise ValueError("context is required and must be a string")
+
+        accepted = json.get("accepted")
+        if accepted is None or not isinstance(accepted, bool):
+            raise ValueError("accepted is required and must be a boolean")
+
+        type_value = json.get("type")
+        if type_value is None or not isinstance(type_value, str):
+            raise ValueError("type is required and must be a string")
+
+        user_text = json.get("userText")
+        if user_text is not None and not isinstance(user_text, str):
+            raise ValueError("userText must be a string if provided")
+
         return cls(
-            context=json.get("context"),
-            accepted=json.get("accepted"),
-            type=json.get("type"),
-            user_text=json.get("userText"),
+            context=context,
+            accepted=accepted,
+            type=type_value,
+            user_text=user_text,
         )
 
 
@@ -236,13 +337,15 @@ class HistoryUpdatedParams:
     def __init__(
         self,
         context: str,
+        timestamp: int,
         url: str,
     ):
         self.context = context
+        self.timestamp = timestamp
         self.url = url
 
     @classmethod
-    def from_json(cls, json: Dict) -> "HistoryUpdatedParams":
+    def from_json(cls, json: dict) -> "HistoryUpdatedParams":
         """Creates a HistoryUpdatedParams instance from a dictionary.
 
         Parameters:
@@ -253,57 +356,317 @@ class HistoryUpdatedParams:
         -------
             HistoryUpdatedParams: A new instance of HistoryUpdatedParams.
         """
+        context = json.get("context")
+        if context is None or not isinstance(context, str):
+            raise ValueError("context is required and must be a string")
+
+        timestamp = json.get("timestamp")
+        if timestamp is None or not isinstance(timestamp, int) or timestamp < 0:
+            raise ValueError("timestamp is required and must be a non-negative integer")
+
+        url = json.get("url")
+        if url is None or not isinstance(url, str):
+            raise ValueError("url is required and must be a string")
+
         return cls(
-            context=json.get("context"),
-            url=json.get("url"),
+            context=context,
+            timestamp=timestamp,
+            url=url,
         )
 
 
-class BrowsingContextEvent:
-    """Base class for browsing context events."""
+class ContextCreated:
+    """Event class for browsingContext.contextCreated event."""
 
-    def __init__(self, event_class: str, **kwargs):
-        self.event_class = event_class
-        self.params = kwargs
+    event_class = "browsingContext.contextCreated"
 
     @classmethod
-    def from_json(cls, json: Dict) -> "BrowsingContextEvent":
-        """Creates a BrowsingContextEvent instance from a dictionary.
+    def from_json(cls, json: dict):
+        if isinstance(json, BrowsingContextInfo):
+            return json
+        return BrowsingContextInfo.from_json(json)
+
+
+class ContextDestroyed:
+    """Event class for browsingContext.contextDestroyed event."""
+
+    event_class = "browsingContext.contextDestroyed"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        if isinstance(json, BrowsingContextInfo):
+            return json
+        return BrowsingContextInfo.from_json(json)
+
+
+class NavigationStarted:
+    """Event class for browsingContext.navigationStarted event."""
+
+    event_class = "browsingContext.navigationStarted"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        if isinstance(json, NavigationInfo):
+            return json
+        return NavigationInfo.from_json(json)
+
+
+class NavigationCommitted:
+    """Event class for browsingContext.navigationCommitted event."""
+
+    event_class = "browsingContext.navigationCommitted"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        if isinstance(json, NavigationInfo):
+            return json
+        return NavigationInfo.from_json(json)
+
+
+class NavigationFailed:
+    """Event class for browsingContext.navigationFailed event."""
+
+    event_class = "browsingContext.navigationFailed"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        if isinstance(json, NavigationInfo):
+            return json
+        return NavigationInfo.from_json(json)
+
+
+class NavigationAborted:
+    """Event class for browsingContext.navigationAborted event."""
+
+    event_class = "browsingContext.navigationAborted"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        if isinstance(json, NavigationInfo):
+            return json
+        return NavigationInfo.from_json(json)
+
+
+class DomContentLoaded:
+    """Event class for browsingContext.domContentLoaded event."""
+
+    event_class = "browsingContext.domContentLoaded"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        if isinstance(json, NavigationInfo):
+            return json
+        return NavigationInfo.from_json(json)
+
+
+class Load:
+    """Event class for browsingContext.load event."""
+
+    event_class = "browsingContext.load"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        if isinstance(json, NavigationInfo):
+            return json
+        return NavigationInfo.from_json(json)
+
+
+class FragmentNavigated:
+    """Event class for browsingContext.fragmentNavigated event."""
+
+    event_class = "browsingContext.fragmentNavigated"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        if isinstance(json, NavigationInfo):
+            return json
+        return NavigationInfo.from_json(json)
+
+
+class DownloadWillBegin:
+    """Event class for browsingContext.downloadWillBegin event."""
+
+    event_class = "browsingContext.downloadWillBegin"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        return DownloadWillBeginParams.from_json(json)
+
+
+class UserPromptOpened:
+    """Event class for browsingContext.userPromptOpened event."""
+
+    event_class = "browsingContext.userPromptOpened"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        return UserPromptOpenedParams.from_json(json)
+
+
+class UserPromptClosed:
+    """Event class for browsingContext.userPromptClosed event."""
+
+    event_class = "browsingContext.userPromptClosed"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        return UserPromptClosedParams.from_json(json)
+
+
+class HistoryUpdated:
+    """Event class for browsingContext.historyUpdated event."""
+
+    event_class = "browsingContext.historyUpdated"
+
+    @classmethod
+    def from_json(cls, json: dict):
+        return HistoryUpdatedParams.from_json(json)
+
+
+@dataclass
+class EventConfig:
+    event_key: str
+    bidi_event: str
+    event_class: type
+
+
+class _EventManager:
+    """Class to manage event subscriptions and callbacks for BrowsingContext."""
+
+    def __init__(self, conn, event_configs: dict[str, EventConfig]):
+        self.conn = conn
+        self.event_configs = event_configs
+        self.subscriptions: dict = {}
+        self._bidi_to_class = {config.bidi_event: config.event_class for config in event_configs.values()}
+        self._available_events = ", ".join(sorted(event_configs.keys()))
+        # Thread safety lock for subscription operations
+        self._subscription_lock = threading.Lock()
+
+    def validate_event(self, event: str) -> EventConfig:
+        event_config = self.event_configs.get(event)
+        if not event_config:
+            raise ValueError(f"Event '{event}' not found. Available events: {self._available_events}")
+        return event_config
+
+    def subscribe_to_event(self, bidi_event: str, contexts: Optional[list[str]] = None) -> None:
+        """Subscribe to a BiDi event if not already subscribed.
 
         Parameters:
-        -----------
-            json: A dictionary containing the event information.
-
-        Returns:
-        -------
-            BrowsingContextEvent: A new instance of BrowsingContextEvent.
+        ----------
+            bidi_event: The BiDi event name.
+            contexts: Optional browsing context IDs to subscribe to.
         """
-        return cls(event_class=json.get("event_class"), **json)
+        with self._subscription_lock:
+            if bidi_event not in self.subscriptions:
+                session = Session(self.conn)
+                self.conn.execute(session.subscribe(bidi_event, browsing_contexts=contexts))
+                self.subscriptions[bidi_event] = []
+
+    def unsubscribe_from_event(self, bidi_event: str) -> None:
+        """Unsubscribe from a BiDi event if no more callbacks exist.
+
+        Parameters:
+        ----------
+            bidi_event: The BiDi event name.
+        """
+        with self._subscription_lock:
+            callback_list = self.subscriptions.get(bidi_event)
+            if callback_list is not None and not callback_list:
+                session = Session(self.conn)
+                self.conn.execute(session.unsubscribe(bidi_event))
+                del self.subscriptions[bidi_event]
+
+    def add_callback_to_tracking(self, bidi_event: str, callback_id: int) -> None:
+        with self._subscription_lock:
+            self.subscriptions[bidi_event].append(callback_id)
+
+    def remove_callback_from_tracking(self, bidi_event: str, callback_id: int) -> None:
+        with self._subscription_lock:
+            callback_list = self.subscriptions.get(bidi_event)
+            if callback_list and callback_id in callback_list:
+                callback_list.remove(callback_id)
+
+    def add_event_handler(self, event: str, callback: Callable, contexts: Optional[list[str]] = None) -> int:
+        event_config = self.validate_event(event)
+
+        callback_id = self.conn.add_callback(event_config.event_class, callback)
+
+        # Subscribe to the event if needed
+        self.subscribe_to_event(event_config.bidi_event, contexts)
+
+        # Track the callback
+        self.add_callback_to_tracking(event_config.bidi_event, callback_id)
+
+        return callback_id
+
+    def remove_event_handler(self, event: str, callback_id: int) -> None:
+        event_config = self.validate_event(event)
+
+        # Remove the callback from the connection
+        self.conn.remove_callback(event_config.event_class, callback_id)
+
+        # Remove from tracking collections
+        self.remove_callback_from_tracking(event_config.bidi_event, callback_id)
+
+        # Unsubscribe if no more callbacks exist
+        self.unsubscribe_from_event(event_config.bidi_event)
+
+    def clear_event_handlers(self) -> None:
+        """Clear all event handlers from the browsing context."""
+        with self._subscription_lock:
+            if not self.subscriptions:
+                return
+
+            session = Session(self.conn)
+
+            for bidi_event, callback_ids in list(self.subscriptions.items()):
+                event_class = self._bidi_to_class.get(bidi_event)
+                if event_class:
+                    # Remove all callbacks for this event
+                    for callback_id in callback_ids:
+                        self.conn.remove_callback(event_class, callback_id)
+
+                    self.conn.execute(session.unsubscribe(bidi_event))
+
+            self.subscriptions.clear()
 
 
 class BrowsingContext:
     """BiDi implementation of the browsingContext module."""
 
-    EVENTS = {
-        "context_created": "browsingContext.contextCreated",
-        "context_destroyed": "browsingContext.contextDestroyed",
-        "dom_content_loaded": "browsingContext.domContentLoaded",
-        "download_will_begin": "browsingContext.downloadWillBegin",
-        "fragment_navigated": "browsingContext.fragmentNavigated",
-        "history_updated": "browsingContext.historyUpdated",
-        "load": "browsingContext.load",
-        "navigation_aborted": "browsingContext.navigationAborted",
-        "navigation_committed": "browsingContext.navigationCommitted",
-        "navigation_failed": "browsingContext.navigationFailed",
-        "navigation_started": "browsingContext.navigationStarted",
-        "user_prompt_closed": "browsingContext.userPromptClosed",
-        "user_prompt_opened": "browsingContext.userPromptOpened",
+    EVENT_CONFIGS = {
+        "context_created": EventConfig("context_created", "browsingContext.contextCreated", ContextCreated),
+        "context_destroyed": EventConfig("context_destroyed", "browsingContext.contextDestroyed", ContextDestroyed),
+        "dom_content_loaded": EventConfig("dom_content_loaded", "browsingContext.domContentLoaded", DomContentLoaded),
+        "download_will_begin": EventConfig(
+            "download_will_begin", "browsingContext.downloadWillBegin", DownloadWillBegin
+        ),
+        "fragment_navigated": EventConfig("fragment_navigated", "browsingContext.fragmentNavigated", FragmentNavigated),
+        "history_updated": EventConfig("history_updated", "browsingContext.historyUpdated", HistoryUpdated),
+        "load": EventConfig("load", "browsingContext.load", Load),
+        "navigation_aborted": EventConfig("navigation_aborted", "browsingContext.navigationAborted", NavigationAborted),
+        "navigation_committed": EventConfig(
+            "navigation_committed", "browsingContext.navigationCommitted", NavigationCommitted
+        ),
+        "navigation_failed": EventConfig("navigation_failed", "browsingContext.navigationFailed", NavigationFailed),
+        "navigation_started": EventConfig("navigation_started", "browsingContext.navigationStarted", NavigationStarted),
+        "user_prompt_closed": EventConfig("user_prompt_closed", "browsingContext.userPromptClosed", UserPromptClosed),
+        "user_prompt_opened": EventConfig("user_prompt_opened", "browsingContext.userPromptOpened", UserPromptOpened),
     }
 
     def __init__(self, conn):
         self.conn = conn
-        self.subscriptions = {}
-        self.callbacks = {}
+        self._event_manager = _EventManager(conn, self.EVENT_CONFIGS)
+
+    @classmethod
+    def get_event_names(cls) -> list[str]:
+        """Get a list of all available event names.
+
+        Returns:
+        -------
+            List[str]: A list of event names that can be used with event handlers.
+        """
+        return list(cls.EVENT_CONFIGS.keys())
 
     def activate(self, context: str) -> None:
         """Activates and focuses the given top-level traversable.
@@ -323,8 +686,8 @@ class BrowsingContext:
         self,
         context: str,
         origin: str = "viewport",
-        format: Optional[Dict] = None,
-        clip: Optional[Dict] = None,
+        format: Optional[dict] = None,
+        clip: Optional[dict] = None,
     ) -> str:
         """Captures an image of the given navigable, and returns it as a Base64-encoded string.
 
@@ -339,7 +702,7 @@ class BrowsingContext:
         -------
             str: The Base64-encoded screenshot.
         """
-        params = {"context": context, "origin": origin}
+        params: dict[str, Any] = {"context": context, "origin": origin}
         if format is not None:
             params["format"] = format
         if clip is not None:
@@ -383,7 +746,7 @@ class BrowsingContext:
         -------
             str: The browsing context ID of the created navigable.
         """
-        params = {"type": type}
+        params: dict[str, Any] = {"type": type}
         if reference_context is not None:
             params["referenceContext"] = reference_context
         if background is not None:
@@ -398,7 +761,7 @@ class BrowsingContext:
         self,
         max_depth: Optional[int] = None,
         root: Optional[str] = None,
-    ) -> List[BrowsingContextInfo]:
+    ) -> list[BrowsingContextInfo]:
         """Returns a tree of all descendent navigables including the given parent itself, or all top-level contexts
         when no parent is provided.
 
@@ -411,7 +774,7 @@ class BrowsingContext:
         -------
             List[BrowsingContextInfo]: A list of browsing context information.
         """
-        params = {}
+        params: dict[str, Any] = {}
         if max_depth is not None:
             params["maxDepth"] = max_depth
         if root is not None:
@@ -434,7 +797,7 @@ class BrowsingContext:
             accept: Whether to accept the prompt.
             user_text: The text to enter in the prompt.
         """
-        params = {"context": context}
+        params: dict[str, Any] = {"context": context}
         if accept is not None:
             params["accept"] = accept
         if user_text is not None:
@@ -445,11 +808,11 @@ class BrowsingContext:
     def locate_nodes(
         self,
         context: str,
-        locator: Dict,
+        locator: dict,
         max_node_count: Optional[int] = None,
-        serialization_options: Optional[Dict] = None,
-        start_nodes: Optional[List[Dict]] = None,
-    ) -> List[Dict]:
+        serialization_options: Optional[dict] = None,
+        start_nodes: Optional[list[dict]] = None,
+    ) -> list[dict]:
         """Returns a list of all nodes matching the specified locator.
 
         Parameters:
@@ -464,7 +827,7 @@ class BrowsingContext:
         -------
             List[Dict]: A list of nodes.
         """
-        params = {"context": context, "locator": locator}
+        params: dict[str, Any] = {"context": context, "locator": locator}
         if max_node_count is not None:
             params["maxNodeCount"] = max_node_count
         if serialization_options is not None:
@@ -480,7 +843,7 @@ class BrowsingContext:
         context: str,
         url: str,
         wait: Optional[str] = None,
-    ) -> Dict:
+    ) -> dict:
         """Navigates a navigable to the given URL.
 
         Parameters:
@@ -504,10 +867,10 @@ class BrowsingContext:
         self,
         context: str,
         background: bool = False,
-        margin: Optional[Dict] = None,
+        margin: Optional[dict] = None,
         orientation: str = "portrait",
-        page: Optional[Dict] = None,
-        page_ranges: Optional[List[Union[int, str]]] = None,
+        page: Optional[dict] = None,
+        page_ranges: Optional[list[Union[int, str]]] = None,
         scale: float = 1.0,
         shrink_to_fit: bool = True,
     ) -> str:
@@ -551,7 +914,7 @@ class BrowsingContext:
         context: str,
         ignore_cache: Optional[bool] = None,
         wait: Optional[str] = None,
-    ) -> Dict:
+    ) -> dict:
         """Reloads a navigable.
 
         Parameters:
@@ -564,7 +927,7 @@ class BrowsingContext:
         -------
             Dict: A dictionary containing the navigation result.
         """
-        params = {"context": context}
+        params: dict[str, Any] = {"context": context}
         if ignore_cache is not None:
             params["ignoreCache"] = ignore_cache
         if wait is not None:
@@ -576,9 +939,9 @@ class BrowsingContext:
     def set_viewport(
         self,
         context: Optional[str] = None,
-        viewport: Optional[Dict] = None,
+        viewport: Optional[dict] = None,
         device_pixel_ratio: Optional[float] = None,
-        user_contexts: Optional[List[str]] = None,
+        user_contexts: Optional[list[str]] = None,
     ) -> None:
         """Modifies specific viewport characteristics on the given top-level traversable.
 
@@ -593,7 +956,7 @@ class BrowsingContext:
         ------
             Exception: If the browsing context is not a top-level traversable.
         """
-        params = {}
+        params: dict[str, Any] = {}
         if context is not None:
             params["context"] = context
         if viewport is not None:
@@ -605,7 +968,7 @@ class BrowsingContext:
 
         self.conn.execute(command_builder("browsingContext.setViewport", params))
 
-    def traverse_history(self, context: str, delta: int) -> Dict:
+    def traverse_history(self, context: str, delta: int) -> dict:
         """Traverses the history of a given navigable by a delta.
 
         Parameters:
@@ -621,51 +984,7 @@ class BrowsingContext:
         result = self.conn.execute(command_builder("browsingContext.traverseHistory", params))
         return result
 
-    def _on_event(self, event_name: str, callback: callable) -> int:
-        """Set a callback function to subscribe to a browsing context event.
-
-        Parameters:
-        ----------
-            event_name: The event to subscribe to.
-            callback: The callback function to execute on event.
-
-        Returns:
-        -------
-            int: callback id
-        """
-        event = BrowsingContextEvent(event_name)
-
-        def _callback(event_data):
-            if event_name == self.EVENTS["context_created"] or event_name == self.EVENTS["context_destroyed"]:
-                info = BrowsingContextInfo.from_json(event_data.params)
-                callback(info)
-            elif event_name == self.EVENTS["download_will_begin"]:
-                params = DownloadWillBeginParams.from_json(event_data.params)
-                callback(params)
-            elif event_name == self.EVENTS["user_prompt_opened"]:
-                params = UserPromptOpenedParams.from_json(event_data.params)
-                callback(params)
-            elif event_name == self.EVENTS["user_prompt_closed"]:
-                params = UserPromptClosedParams.from_json(event_data.params)
-                callback(params)
-            elif event_name == self.EVENTS["history_updated"]:
-                params = HistoryUpdatedParams.from_json(event_data.params)
-                callback(params)
-            else:
-                # For navigation events
-                info = NavigationInfo.from_json(event_data.params)
-                callback(info)
-
-        callback_id = self.conn.add_callback(event, _callback)
-
-        if event_name in self.callbacks:
-            self.callbacks[event_name].append(callback_id)
-        else:
-            self.callbacks[event_name] = [callback_id]
-
-        return callback_id
-
-    def add_event_handler(self, event: str, callback: callable, contexts: Optional[List[str]] = None) -> int:
+    def add_event_handler(self, event: str, callback: Callable, contexts: Optional[list[str]] = None) -> int:
         """Add an event handler to the browsing context.
 
         Parameters:
@@ -678,24 +997,7 @@ class BrowsingContext:
         -------
             int: callback id
         """
-        try:
-            event_name = self.EVENTS[event]
-        except KeyError:
-            raise Exception(f"Event {event} not found")
-
-        callback_id = self._on_event(event_name, callback)
-
-        if event_name in self.subscriptions:
-            self.subscriptions[event_name].append(callback_id)
-        else:
-            params = {"events": [event_name]}
-            if contexts is not None:
-                params["browsingContexts"] = contexts
-            session = Session(self.conn)
-            self.conn.execute(session.subscribe(**params))
-            self.subscriptions[event_name] = [callback_id]
-
-        return callback_id
+        return self._event_manager.add_event_handler(event, callback, contexts)
 
     def remove_event_handler(self, event: str, callback_id: int) -> None:
         """Remove an event handler from the browsing context.
@@ -705,28 +1007,8 @@ class BrowsingContext:
             event: The event to unsubscribe from.
             callback_id: The callback id to remove.
         """
-        try:
-            event_name = self.EVENTS[event]
-        except KeyError:
-            raise Exception(f"Event {event} not found")
-
-        event = BrowsingContextEvent(event_name)
-
-        self.conn.remove_callback(event, callback_id)
-        self.subscriptions[event_name].remove(callback_id)
-        if len(self.subscriptions[event_name]) == 0:
-            params = {"events": [event_name]}
-            session = Session(self.conn)
-            self.conn.execute(session.unsubscribe(**params))
-            del self.subscriptions[event_name]
+        self._event_manager.remove_event_handler(event, callback_id)
 
     def clear_event_handlers(self) -> None:
         """Clear all event handlers from the browsing context."""
-        for event_name in self.subscriptions:
-            event = BrowsingContextEvent(event_name)
-            for callback_id in self.subscriptions[event_name]:
-                self.conn.remove_callback(event, callback_id)
-            params = {"events": [event_name]}
-            session = Session(self.conn)
-            self.conn.execute(session.unsubscribe(**params))
-        self.subscriptions = {}
+        self._event_manager.clear_event_handlers()
