@@ -19,13 +19,15 @@
 import os
 import time
 from platform import system
-from subprocess import STDOUT
-from subprocess import Popen
+from subprocess import DEVNULL, STDOUT, Popen
+
+from typing_extensions import deprecated
 
 from selenium.common.exceptions import WebDriverException
 from selenium.webdriver.common import utils
 
 
+@deprecated("Use binary_location property in Firefox Options to set location")
 class FirefoxBinary:
     NO_FOCUS_LIBRARY_NAME = "x_ignore_nofocus.so"
 
@@ -41,7 +43,7 @@ class FirefoxBinary:
         self._start_cmd = firefox_path
         # We used to default to subprocess.PIPE instead of /dev/null, but after
         # a while the pipe would fill up and Firefox would freeze.
-        self._log_file = log_file or open(os.devnull, "wb")
+        self._log_file = log_file or DEVNULL
         self.command_line = None
         self.platform = system().lower()
         if not self._start_cmd:
@@ -120,12 +122,9 @@ class FirefoxBinary:
 
     def _find_exe_in_registry(self):
         try:
-            from _winreg import HKEY_CURRENT_USER
-            from _winreg import HKEY_LOCAL_MACHINE
-            from _winreg import OpenKey
-            from _winreg import QueryValue
+            from _winreg import HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE, OpenKey, QueryValue
         except ImportError:
-            from winreg import OpenKey, QueryValue, HKEY_LOCAL_MACHINE, HKEY_CURRENT_USER
+            from winreg import HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE, OpenKey, QueryValue
         import shlex
 
         keys = (
@@ -161,7 +160,7 @@ class FirefoxBinary:
             start_cmd = self.which(ffname)
             # use hardcoded path if nothing else was found by which()
             if not start_cmd:
-                start_cmd = "/Applications/Firefox.app/Contents/MacOS/firefox-bin"
+                start_cmd = "/Applications/Firefox.app/Contents/MacOS/firefox"
             # fallback to homebrew installation for mac users
             if not os.path.exists(start_cmd):
                 start_cmd = os.path.expanduser("~") + start_cmd
