@@ -15,12 +15,12 @@
 # specific language governing permissions and limitations
 # under the License.
 import base64
-import logging
 import os
 import warnings
 import zipfile
 from contextlib import contextmanager
 from io import BytesIO
+from typing import Optional
 
 from selenium.webdriver.common.driver_finder import DriverFinder
 from selenium.webdriver.remote.webdriver import WebDriver as RemoteWebDriver
@@ -29,8 +29,6 @@ from .options import Options
 from .remote_connection import FirefoxRemoteConnection
 from .service import Service
 
-logger = logging.getLogger(__name__)
-
 
 class WebDriver(RemoteWebDriver):
     """Controls the GeckoDriver and allows you to drive the browser."""
@@ -40,8 +38,8 @@ class WebDriver(RemoteWebDriver):
 
     def __init__(
         self,
-        options: Options = None,
-        service: Service = None,
+        options: Optional[Options] = None,
+        service: Optional[Service] = None,
         keep_alive: bool = True,
     ) -> None:
         """Creates a new instance of the Firefox driver. Starts the service and
@@ -61,7 +59,7 @@ class WebDriver(RemoteWebDriver):
             options.binary_location = finder.get_browser_path()
             options.browser_version = None
 
-        self.service.path = finder.get_driver_path()
+        self.service.path = self.service.env_path() or finder.get_driver_path()
         self.service.start()
 
         executor = FirefoxRemoteConnection(
@@ -125,7 +123,7 @@ class WebDriver(RemoteWebDriver):
         :Usage:
             ::
 
-                driver.install_addon('/path/to/firebug.xpi')
+                driver.install_addon("/path/to/firebug.xpi")
         """
 
         if os.path.isdir(path):
@@ -134,7 +132,7 @@ class WebDriver(RemoteWebDriver):
             path = os.path.normpath(path)
             # account for trailing slash that will be added by os.walk()
             path_root = len(path) + 1
-            with zipfile.ZipFile(fp, "w", zipfile.ZIP_DEFLATED) as zipped:
+            with zipfile.ZipFile(fp, "w", zipfile.ZIP_DEFLATED, strict_timestamps=False) as zipped:
                 for base, _, files in os.walk(path):
                     for fyle in files:
                         filename = os.path.join(base, fyle)
@@ -153,7 +151,7 @@ class WebDriver(RemoteWebDriver):
         :Usage:
             ::
 
-                driver.uninstall_addon('addon@foo.com')
+                driver.uninstall_addon("addon@foo.com")
         """
         self.execute("UNINSTALL_ADDON", {"id": identifier})
 
@@ -169,7 +167,7 @@ class WebDriver(RemoteWebDriver):
         :Usage:
             ::
 
-                driver.get_full_page_screenshot_as_file('/Screenshots/foo.png')
+                driver.get_full_page_screenshot_as_file("/Screenshots/foo.png")
         """
         if not filename.lower().endswith(".png"):
             warnings.warn(
@@ -198,7 +196,7 @@ class WebDriver(RemoteWebDriver):
         :Usage:
             ::
 
-                driver.save_full_page_screenshot('/Screenshots/foo.png')
+                driver.save_full_page_screenshot("/Screenshots/foo.png")
         """
         return self.get_full_page_screenshot_as_file(filename)
 
@@ -223,3 +221,9 @@ class WebDriver(RemoteWebDriver):
                 driver.get_full_page_screenshot_as_base64()
         """
         return self.execute("FULL_PAGE_SCREENSHOT")["value"]
+
+    def download_file(self, *args, **kwargs):
+        raise NotImplementedError
+
+    def get_downloadable_files(self, *args, **kwargs):
+        raise NotImplementedError
