@@ -40,6 +40,8 @@ from selenium.common.exceptions import (
 )
 from selenium.webdriver.common.bidi.browser import Browser
 from selenium.webdriver.common.bidi.browsing_context import BrowsingContext
+from selenium.webdriver.common.bidi.emulation import Emulation
+from selenium.webdriver.common.bidi.input import Input
 from selenium.webdriver.common.bidi.network import Network
 from selenium.webdriver.common.bidi.permissions import Permissions
 from selenium.webdriver.common.bidi.script import Script
@@ -270,6 +272,8 @@ class WebDriver(BaseWebDriver):
         self._storage = None
         self._webextension = None
         self._permissions = None
+        self._emulation = None
+        self._input = None
         self._devtools = None
 
     def __repr__(self):
@@ -1209,7 +1213,10 @@ class WebDriver(BaseWebDriver):
             raise RuntimeError("CDP support for Firefox has been removed. Please switch to WebDriver BiDi.")
         self._websocket_connection = WebSocketConnection(ws_url)
         targets = self._websocket_connection.execute(self._devtools.target.get_targets())
-        target_id = targets[0].target_id
+        for target in targets:
+            if target.target_id == self.current_window_handle:
+                target_id = target.target_id
+                break
         session = self._websocket_connection.execute(self._devtools.target.attach_to_target(target_id, True))
         self._websocket_connection.session_id = session
         return self._devtools, self._websocket_connection
@@ -1230,7 +1237,10 @@ class WebDriver(BaseWebDriver):
         devtools = cdp.import_devtools(version)
         async with cdp.open_cdp(ws_url) as conn:
             targets = await conn.execute(devtools.target.get_targets())
-            target_id = targets[0].target_id
+            for target in targets:
+                if target.target_id == self.current_window_handle:
+                    target_id = target.target_id
+                    break
             async with conn.open_session(target_id) as session:
                 yield BidiConnection(session, cdp, devtools)
 
@@ -1390,6 +1400,51 @@ class WebDriver(BaseWebDriver):
 
         return self._webextension
 
+    @property
+    def emulation(self):
+        """Returns an emulation module object for BiDi emulation commands.
+
+        Returns:
+        --------
+        Emulation: an object containing access to BiDi emulation commands.
+
+        Examples:
+        ---------
+        >>> from selenium.webdriver.common.bidi.emulation import GeolocationCoordinates
+        >>> coordinates = GeolocationCoordinates(37.7749, -122.4194)
+        >>> driver.emulation.set_geolocation_override(coordinates=coordinates, contexts=[context_id])
+        """
+        if not self._websocket_connection:
+            self._start_bidi()
+
+        if self._emulation is None:
+            self._emulation = Emulation(self._websocket_connection)
+
+        return self._emulation
+
+    @property
+    def input(self):
+        """Returns an input module object for BiDi input commands.
+
+        Returns:
+        --------
+        Input: an object containing access to BiDi input commands.
+
+        Examples:
+        ---------
+        >>> from selenium.webdriver.common.bidi.input import KeySourceActions, KeyDownAction, KeyUpAction
+        >>> key_actions = KeySourceActions(id="keyboard", actions=[KeyDownAction(value="a"), KeyUpAction(value="a")])
+        >>> driver.input.perform_actions(driver.current_window_handle, [key_actions])
+        >>> driver.input.release_actions(driver.current_window_handle)
+        """
+        if not self._websocket_connection:
+            self._start_bidi()
+
+        if self._input is None:
+            self._input = Input(self._websocket_connection)
+
+        return self._input
+
     def _get_cdp_details(self):
         import json
 
@@ -1399,7 +1454,7 @@ class WebDriver(BaseWebDriver):
         try:
             if self.caps.get("browserName") == "chrome":
                 debugger_address = self.caps.get("goog:chromeOptions").get("debuggerAddress")
-            elif self.caps.get("browserName") == "MicrosoftEdge":
+            elif self.caps.get("browserName") in ("MicrosoftEdge", "webview2"):
                 debugger_address = self.caps.get("ms:edgeOptions").get("debuggerAddress")
         except AttributeError:
             raise WebDriverException("Can't get debugger address.")
